# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest

from grymp.processors.feature import Feature


class TestFeature(unittest.TestCase):

    def test_translate_name_no_year(self):
        with self.assertRaises(ValueError):
            Feature.translate_name('foobar')

    def test_translate_name_substitution(self):
        self.assertEqual(Feature.translate_name(
            'Dont.Foo.2222.Bluray.1080p.DTS-HD.x264-Grym'),
                         'Don\'t Foo')

    def test_translate_name_dash(self):
        self.assertEqual(Feature.translate_name(
            'Foo.Bar-Baz.Bat.2245.Bluray.1080p.DTS-HD.x264-Grym'),
                         'Foo Bar - Baz Bat')

    def test_translate_name_dot(self):
        self.assertEqual(Feature.translate_name(
            'Foo.Bar.Baz.1995.Bluray.1080p.DTS-HD.x264-Grym'),
                         'Foo Bar Baz')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest
import sys
import os
import contextlib

from grymp import __main__ as main


@contextlib.contextmanager
def _suppress_stderr():
    save_stderr = sys.stderr
    try:
        sys.stderr = open(os.devnull, 'w')
        yield
    finally:
        sys.stderr.close()
        sys.stderr = save_stderr


class TestParseArgs(unittest.TestCase):

    _BASE = ['/base']
    BASE_ARGV = ['grymp'] + _BASE

    def test_version(self):
        with self.assertRaises(SystemExit), _suppress_stderr():
            main._parse_args(['-V'])

    def test_verbosity_implicit(self):
        self.assertEqual(main._parse_args(self.BASE_ARGV).verbosity, 0)

    def test_verbosity_count(self):
        self.assertEqual(main._parse_args(self.BASE_ARGV +
                                          ['-vvvv']).verbosity,
                         4)

    def test_interactive_missing(self):
        self.assertFalse(main._parse_args(self.BASE_ARGV).interactive)

    def test_interactive(self):
        self.assertTrue(
            main._parse_args(self.BASE_ARGV + ['-i']).interactive)

    def test_overwrite_missing(self):
        self.assertFalse(main._parse_args(self.BASE_ARGV).overwrite)

    def test_overwrite(self):
        self.assertTrue(
            main._parse_args(self.BASE_ARGV + ['-o']).overwrite)

    def test_keep_missing(self):
        self.assertFalse(main._parse_args(self.BASE_ARGV).keep)

    def test_keep(self):
        self.assertTrue(
            main._parse_args(self.BASE_ARGV + ['-k']).keep)

    def test_feature_missing(self):
        self.assertIsNone(main._parse_args(self.BASE_ARGV).feature)

    def test_feature(self):
        feature = 'abc'
        self.assertEqual(
            main._parse_args(self.BASE_ARGV + ['-f', feature]).feature,
            feature)

    def test_extras_missing(self):
        self.assertIsNone(main._parse_args(self.BASE_ARGV).extras)

    def test_extras(self):
        extras = 'def'
        self.assertEqual(
            main._parse_args(self.BASE_ARGV + ['-e', extras]).extras,
            extras)

    def test_base(self):
        self.assertEqual(main._parse_args(self.BASE_ARGV).base, self._BASE)

    def test_base_multiple(self):
        args = ['second_base']
        self.assertEqual(main._parse_args(self.BASE_ARGV + args).base,
                         self._BASE + args)

    # TODO would be great to be able to ensure that either -f or -e is passed
    #      at the argparse level, but it doesn't seem to be able to handle it:
    #      http://stackoverflow.com/questions/6722936
    #      tested below for the time being


class TestMain(unittest.TestCase):

    def test_feature_or_model(self):
        with _suppress_stderr():
            self.assertEqual(main.main(TestParseArgs.BASE_ARGV), 1)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest
import sys
import six
import logging

from grymp import util


class TestPrintError(unittest.TestCase):

    _MESSAGE = 'test message'

    def test_stream(self):
        try:
            sys.stderr = six.StringIO()
            util.print_error(self._MESSAGE)
            self.assertEquals(sys.stderr.getvalue(), self._MESSAGE + '\n')
        finally:
            sys.stderr = sys.__stderr__


class TestDecodeCliArg(unittest.TestCase):

    _ARG_VALUE = 'test_arg_value'

    def test_empty(self):
        with self.assertRaises(ValueError):
            util.decode_cli_arg(None)

    @unittest.skipUnless(sys.version_info.major == 2,
                         'Only applies to Python 2')
    def test_valid_2(self):
        self.assertEqual(
            util.decode_cli_arg(
                self._ARG_VALUE.encode(sys.getfilesystemencoding())),
            self._ARG_VALUE)

    @unittest.skipUnless(sys.version_info.major == 3,
                         'Only applies to Python 3')
    def test_valid_3(self):
        self.assertEqual(util.decode_cli_arg(self._ARG_VALUE), self._ARG_VALUE)


class TestLogLevelFromVerbosity(unittest.TestCase):

    def test_warning(self):
        self.assertEqual(util.log_level_from_vebosity(0), logging.WARNING)

    def test_info(self):
        self.assertEqual(util.log_level_from_vebosity(1), logging.INFO)

    def test_debug(self):
        self.assertEqual(util.log_level_from_vebosity(2), logging.DEBUG)
        self.assertEqual(util.log_level_from_vebosity(None), logging.DEBUG)


class TestRchop(unittest.TestCase):

    _PREFIX = 'foo'
    _SUFFIX = 'bar'

    def test_suffix_removed(self):
        self.assertEqual(util.rchop(self._PREFIX + self._SUFFIX, self._SUFFIX),
                         self._PREFIX)

    def test_unaffected(self):
        string = self._PREFIX + self._SUFFIX + '!'
        self.assertEqual(util.rchop(string, self._SUFFIX), string)


# this class makes assumptions, but fairly safe ones
class TestSameFilesystem(unittest.TestCase):

    def test_same_fs_exists(self):
        self.assertTrue(util.same_filesystem('/', '/root'))

    def test_same_fs_absent(self):
        self.assertTrue(util.same_filesystem('/missing', '/root/missing'))

    def test_different_fs_exists(self):
        self.assertFalse(util.same_filesystem('/', '/dev/shm'))

    def test_different_fs_absent(self):
        self.assertFalse(util.same_filesystem('/missing', '/dev/shm/missing'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import re
import logging

from grymp.processors import processor

logger = logging.getLogger(__name__)


class Feature(processor.Processor):
    """
    Handles the processing of a release's feature.
    """

    # the extension of the feature
    _EXTENSION = 'mkv'

    @staticmethod
    def translate_name(name):
        """
        Guess the feature name from the file name.

        :param name: The feature file name.
        :return: Our best guess at the original name.
        """

        _SUBSTITUTIONS = {
            'Dont': 'Don\'t'
        }

        result = re.match(r'(.+?)\.\d{4}\.', name)
        if not result:
            raise ValueError('Unable to identify any name')

        title = result.group(1)
        title = title.replace('-', ' - ')
        title = title.replace('.', ' ')
        terms = title.split(' ')
        terms = [_SUBSTITUTIONS[term] if term in _SUBSTITUTIONS else term
                 for term in terms]
        return ' '.join(terms)

    def process(self, base, name, feature_dir):
        """
        Process the feature in the release directory.

        :param base: The release directory, containing the feature.
        :param name: The name of the feature.
        :param feature_dir: The directory to move the feature to.
        :return: True if the feature was transferred successfully; False if the
                 user cancelled a prompt.
        """
        path = self._find_entry(base, self._EXTENSION)
        logger.debug('Located feature at %s', path)
        return self._fs.transfer(
            path, os.path.join(feature_dir, name + '.' + self._EXTENSION))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function
import os
import sys
import logging

import six
from six.moves import input


def print_error(msg):
    """
    Print a string to stderr.

    :param msg: The message to print.
    """
    print(msg, file=sys.stderr)


def decode_cli_arg(arg):
    """
    Turn a bytestring provided by `argparse` into unicode.

    :param arg: The bytestring to decode.
    :return: The argument as a unicode object.
    :raises ValueError: If arg is None.
    """
    if arg is None:
        raise ValueError('Argument cannot be None')

    if sys.version_info.major == 3:
        # already decoded
        return arg

    return arg.decode(sys.getfilesystemencoding())


def log_level_from_vebosity(verbosity):
    """
    Get the `logging` module log level from a verbosity.

    :param verbosity: The number of times the `-v` option was specified.
    :return: The corresponding log level.
    """
    if verbosity == 0:
        return logging.WARNING
    if verbosity == 1:
        return logging.INFO
    return logging.DEBUG


def rchop(string, suffix):
    """
    Remove a suffix from a string if it appears.

    :param string: The string to chip.
    :param suffix: The suffix to remove from the string.
    :return: The string with the suffix removed from the end, if it exists.
    """
    # http://stackoverflow.com/a/3663505
    if string.endswith(suffix):
        return string[:-len(suffix)]
    return string


def same_filesystem(*args):
    """
    Ascertain whether a number of paths are all located on the same filesystem.

    :param args: Two or more paths to check; if one argument is passed, the
                 result is True by definition. If a path does not exist, this
                 function will ascend the directory hierarchy to the first one
                 that does exist and compare based on that.
    :return: True if the supplied paths are all on the same filesystem; False
             otherwise.
    """

    def _reduce_until_exists(path):
        """
        Ascend the directory tree from a path until we reach a directory that
        exists.

        :param path: The path to reduce.
        :return: The longest existing prefix of that path, representing a valid
                 directory, or file if unchanged.
        """
        while not os.path.exists(path):
            path = os.path.dirname(path)
        return path

    # inspired by http://stackoverflow.com/a/249796
    return len(set([os.lstat(_reduce_until_exists(path)).st_dev
                    for path in args])) == 1


def string_prompt(prompt, default=None):
    """
    Ask the user for string input, obtaining a single line answer.

    Default   | Input     | Return
    ==========|===========|============
    None      | Empty     | <re-prompt>
    None      | Non-empty | The input
    None      | C-d       | EOFError
    ----------|-----------|------------
    Non-empty | Empty     | The default
    Non-empty | Non-empty | The input
    Non-empty | C-d       | The default

    :param prompt: The prompt prefix.
    :param default: The default response if the user hits enter. If this is
                    `None`, the user will be repeatedly asked until their input
                    is non-empty, or C-d is hit.
    :return: The user's input. Guaranteed not to be `None`.
    :raises EOFError: If the user hits C-d and default is `None` (if default is
                      not `None`, default will be returned normally instead).
    """
    prompt_string = '{0}{1}: '.format(
        prompt,
        ' [{0}]'.format(default) if default else '')

    while True:
        try:
            response = six.moves.input(prompt_string).strip()
            if response:
                return response
            if default:
                return default

            # no response, and no default
            print_error('There is no default; please provide a response')
        except EOFError:
            if default is None:
                raise
            return default


def binary_prompt(question, default=True):
    """
    Ask the user a question and obtain a yes/no answer.

    Default | Input    | Result     | Return
    ========|==========|============|============
    True    | y/ye/yes | True       | True
    True    | n/no     | False      | False
    True    | ''       | None       | True
    True    | blah     | ValueError | <re-prompt>
    True    | C-d      | <N/A>      | True
    --------|----------|------------|------------
    False   | y/ye/yes | True       | True
    False   | n/no     | False      | False
    False   | ''       | None       | False
    False   | blah     | ValueError | <re-prompt>
    False   | C-d      | <N/A>      | False
    --------|----------|------------|------------
    None    | y/ye/yes | True       | True
    None    | n/no     | False      | False
    None    | ''       | None       | <re-prompt>
    None    | blah     | ValueError | <re-prompt>
    None    | C-d      | <N/A>      | EOFError

    :param question: What to ask the user.
    :param default: How to interpret the user's response if they do not enter
                    anything (True/False). 'None' means force the user to make
                    a choice - keep prompting until they do.
    :return: True if the user said yes; false if they said no. Guaranteed not to
             be None.
    :raises ValueError: If an invalid default is passed.
    :raises EOFError: If the user hits C-d and default is None (if default is
                      True/False, that would be returned normally instead).
    """
    _PROMPTS = {
        True: '[Y/n]',
        False: '[y/N]',
        None: '[y/n]'
    }
    _RESPONSES = {
        True: ['y', 'ye', 'yes'],
        False: ['n', 'no'],
        None: ['']
    }

    def _parse_result(response):
        """
        Retrieve the result that corresponds to a response.

        :param response: One of the values in _RESPONSES.
        :return: True, False or None.
        :raises ValueError: If the response is not mapped to a result.
        """
        for result_, responses in six.iteritems(_RESPONSES):
            if response in responses:
                return result_
        raise ValueError('Invalid response received')

    if default not in _PROMPTS.keys():
        raise ValueError('Default prompt result cannot be {0}'.format(default))

    while True:
        print('{0} {1} '.format(question, _PROMPTS[default]), end='')
        try:
            selection = input().lower()
            result = _parse_result(selection)

            if result is None:
                if default is None:
                    # must get an answer
                    print_error('There is no default; please specify y or n')
                    continue

                return default

            return result
        except EOFError:
            # C-d pressed
            if default is None:
                # the user is not expecting None; we can only give them an
                # exception
                raise
            return default
        except ValueError:
            # invalid input that is not empty
            print_error('Please respond with y or n')
            continue



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import shutil
import logging
import six

from grymp import util

logger = logging.getLogger(__name__)


@six.python_2_unicode_compatible
class Transferer(object):
    """
    Oversees the copying and moving of files.
    """

    def __init__(self, confirm_all=False, confirm_overwrite=True,
                 keep_originals=False, dir_mode=0o755):
        """
        Initialise a new transferer instance.

        :param confirm_all: Whether to prompt before all filesystem operations.
        :param confirm_overwrite: Whether to prompt before overwriting files.
        :param keep_originals: Whether to delete copied files.
        :param dir_mode: The mode to use for new directories.
        """
        self._confirm_all = confirm_all
        self._confirm_overwrite = confirm_overwrite
        self._keep_originals = keep_originals
        self._dir_mode = dir_mode

    def _validate(self, dest, message):
        """
        Perform checks prior to a move or copy operation.

        :param dest: The destination file path.
        :param message: The message to use to confirm the operation.
        :return: True if permission is granted; false otherwise.
        """
        # if we've been asked to, get permission to perform the filesystem
        # operation
        if self._confirm_all and not util.binary_prompt(message):
            return False

        # if the destination exists and we've been asked to, get permission to
        # overwrite it
        if os.path.exists(dest) and \
                self._confirm_overwrite and \
                not util.binary_prompt('Overwrite {0}?'.format(dest)):
            return False

        return True

    def move(self, src, dest):
        """
        Move a file, possibly prompting.

        :param src: The source file path.
        :param dest: The path to move the file to.
        :return: True if the file was moved successfully; False if the user
                 rejected the confirmation.
        :raises OSError: If a filesystem-related error prevented the move.
        """
        if not self._validate(dest, 'Move {0} to {1}?'.format(src, dest)):
            return False

        logger.info('Moving %s to %s', src, dest)
        os.rename(src, dest)
        return True

    def copy_delete(self, src, dest):
        """
        Copy a file, possibly prompting, then possibly delete the original,
        possibly prompting again.

        :param src: The source file path.
        :param dest: The path to copy the file to.
        :return: True if the file was copied successfully; False if the user
                 rejected the copy confirmation. This will be True even if the
                 user decided to reject the delete confirmation.
        :raises OSError: If a filesystem-related error prevented the copy.
        """
        if not self._validate(dest, 'Copy {0} to {1}?'.format(src, dest)):
            return False

        logger.info('Copying %s to %s', src, dest)
        # TODO show progress and make cancelable
        shutil.copy2(src, dest)

        if not self._keep_originals:
            if self._confirm_all and not util.binary_prompt(
                        'Delete copied file {0}?'.format(src)):
                return False

            try:
                logger.info('Deleting %s', src)
                os.remove(src)
            except IOError as e:
                # ah well
                util.print_error(
                    'Could not delete file {0}: {1}'.format(src, str(e)))

        return True

    def transfer(self, src, dest):
        """
        Get a file to another place, possibly prompting the user for
        confirmation. If the source and destination are on the same filesystem,
        the file will be moved. Otherwise it will be copied using rsync; in this
        case, the user may also be prompted to delete the original.

        :param src: The path to the source file.
        :param dest: The path to copy or move the file to.
        :return: True if the file was transferred successfully; False if the
                 user cancelled a prompt.
        :raises OSError: If a filesystem-related error occurred.
        """

        logger.debug('Asked to transfer %s to %s', src, dest)

        # Use dirname of dest as the actual dest may not exist so we can't do
        # the same filesystem check
        if util.same_filesystem(src, os.path.dirname(dest)):
            logger.debug('Moving as they are on the same filesystem')
            return self.move(src, dest)

        return self.copy_delete(src, dest)

    def maybe_mkdir(self, path):
        """
        Create a new directory if it does not already exist.

        :param path: The path of the directory.
        :return: True if the directory exists or was created; False if the
                 user cancelled a prompt.
        :raises OSError: If a filesystem-related error occurred.
        """
        if os.path.exists(path):
            os.chmod(path, self._dir_mode)
            return True

        if self._confirm_all and \
                not util.binary_prompt('Create directory {0}?'.format(path)):
            return False

        logger.info('Creating directory %s', path)
        os.mkdir(path, self._dir_mode)
        return True

    def __str__(self):
        return 'Transferer(' \
               'confirm_all: {0}, ' \
               'confirm_overwrite: {1}, ' \
               'delete_originals: {2})'.format(
                    self._confirm_all,
                    self._confirm_overwrite,
                    self._keep_originals)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import glob
import logging

logger = logging.getLogger(__name__)


class Processor(object):
    """
    Represents a processor that handles a part of a release.
    """

    def __init__(self, fs):
        """
        Initialise a new processor instance.

        :param fs: A Transferer instance to use to execute filesystem
                     operations.
        """
        self._fs = fs

    @staticmethod
    def _find_entry(directory, suffix):
        """
        Find a file or directory with a suffix within a directory.

        :param directory: The directory to search within.
        :param suffix: The file or directory suffix.
        :return: The entry path if found.
        :raises IOError: If there is not exactly 1 matching entry.
        """
        matches = glob.glob('{0}/*{1}'.format(directory, suffix))
        if len(matches) != 1:
            raise IOError(
                'Failed to identify entry ending with {0} in {1}'.format(
                    suffix, directory))
        return matches[0]

    @staticmethod
    def _find_file(directory, extension):
        """
        Find the file with an extension in a directory.

        :param directory: The directory in which to look for the file.
        :param extension: The extension of the file.
        :return: The file path if found.
        :raises IOError: If there is not exactly 1 matching file.
        """
        path = Processor._find_entry(directory, '.' + extension)
        if not os.path.isfile(path):
            raise IOError('Found entry {0} is not a file'.format(path))
        return path



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division
import os
import sys
import argparse
import logging

import grymp
from grymp import util, transferer
from grymp.processors import extras, feature

logger = logging.getLogger(__name__)


def _parse_args(args):
    """
    Interpret command line arguments.

    :param args: `sys.argv`
    :return: The populated argparse namespace.
    """

    parser = argparse.ArgumentParser(prog='grymp',
                                     description='Automate the processing of '
                                                 'Grym/vonRicht releases')
    parser.add_argument('-V', '--version',
                        action='version',
                        version='%(prog)s ' + grymp.__version__)
    parser.add_argument('-v', '--verbosity',
                        help='increase output verbosity',
                        action='count',
                        default=0)
    parser.add_argument('-i', '--interactive',
                        help='prompt before each operation affecting the '
                             'filesystem',
                        action='store_true')
    parser.add_argument('-o', '--overwrite',
                        help='overwrite files without asking',
                        action='store_true')
    parser.add_argument('-k', '--keep',
                        help='keep original files; only has an effect if '
                             'they were copied because the destination was on '
                             'a different filesystem',
                        action='store_true')
    parser.add_argument('-f', '--feature',
                        help='marshal the feature into this directory (it must '
                             'exist)')
    parser.add_argument('-e', '--extras',
                        help='marshal the extras into a new subdirectory '
                             'within this directory, named after the feature')
    parser.add_argument('base',
                        help='one or more root directories of releases, '
                             'each containing a feature',
                        nargs='+',
                        type=util.decode_cli_arg)
    return parser.parse_args(args[1:])


def main(args):
    """
    grymp's entry point.

    :param args: Command-line arguments, with the program in position 0.
    """

    args = _parse_args(args)

    # sort out logging output and level
    level = util.log_level_from_vebosity(args.verbosity)
    root = logging.getLogger()
    root.setLevel(level)
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(level)
    handler.setFormatter(logging.Formatter('%(levelname)s %(message)s'))
    root.addHandler(handler)

    logger.debug(args)

    if not args.feature and not args.extras:
        util.print_error('At least one of -f and -e must be specified, '
                         'otherwise there is nothing to do!')
        return 1

    fs = transferer.Transferer(
        confirm_all=args.interactive,
        confirm_overwrite=not args.overwrite,
        keep_originals=args.keep)
    logger.debug('Created transferer: %s', fs)

    try:
        for base in args.base:
            basename = os.path.basename(base)
            name = util.string_prompt(
                'Name of feature in {0}'.format(basename),
                feature.Feature.translate_name(basename))
            if args.feature:
                feature.Feature(fs).process(base, name, args.feature)
            if args.extras:
                extras.Extras(fs).process(base, name, args.extras)
    except IOError as e:
        util.print_error(str(e))
        return 2
    except EOFError:
        return 3

    return 0


def main_cli():
    """
    grymp's command-line entry point.

    :return: The return code of the program.
    """
    status = main(sys.argv)
    logger.debug('Returning exit status %d', status)
    return status


if __name__ == '__main__':
    sys.exit(main_cli())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest

from grymp.processors.extras import Extras


class TestExtras(unittest.TestCase):

    def test_translate_name_directory(self):
        self.assertEqual(
            Extras._translate_name('A.b.C.d-E.F-Grym'),
            'A b C d - E F')

    def test_translate_name_file(self):
        self.assertEqual(
            Extras._translate_name('Foo.Bar.baz.Bat.(Blim.Blom)-Grym.mkv'),
            'Foo Bar baz Bat (Blim Blom).mkv')

    def test_translate_name_file_blacklist(self):
        self.assertEqual(
            Extras._translate_name('Foo.Bar.720p.TrueHD.Atmos.DD-5.1.(Blim.Blom'
                                   ')-Grym.mkv'),
            'Foo Bar (Blim Blom).mkv')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import logging

from grymp import util
from grymp.processors import processor

logger = logging.getLogger(__name__)


class Extras(processor.Processor):
    """
    Handles the processing of release extras.
    """

    _EXTRAS_SUFFIX = 'Extras-Grym'

    @staticmethod
    def _translate_name(name):
        """
        Attempts to resurrect the original file or directory name from the
        Grym/vR version.

        :param name: The mutated name.
        :return: Our best guess at the original name.
        """

        # substrings to remove from any name
        _FRAGMENT_BLACKLIST = ['TrueHD', 'Atmos', 'DD-5.1', '720p', '1080p',
                               'Bluray', 'DTS-HD', 'x264']

        if name.endswith('-Grym'):
            # is directory
            extension = ''
        else:
            # is file
            name, extension = os.path.splitext(name)

        name = util.rchop(name, '-Grym')
        for fragment in _FRAGMENT_BLACKLIST:
            name = name.replace(fragment + '.', '')
        name = name.replace('.', ' ')
        name = name.replace('-', ' - ')
        return name + extension

    def _process_recursive(self, local_dir, remote_dir):
        """
        Recursively transfer extras in a local directory to a remote one.

        :param local_dir: The source directory.
        :param remote_dir: The destination directory.
        :return: True if the transfer finished normally; N.B. not all
                 directories may have been copied. False if the user rejected
                 the creation of a directory - in which case the process cannot
                 continue.
        """
        # we don't use os.walk() as we would have to translate intermediate
        # paths over and over again
        for name in os.listdir(local_dir):
            local_path = os.path.join(local_dir, name)
            translated_name = self._translate_name(name)
            remote_path = os.path.join(remote_dir, translated_name)
            if os.path.isdir(local_path):
                if not self._fs.maybe_mkdir(remote_path) or \
                        not self._process_recursive(local_path, remote_path):
                    # we can't continue if the directory structure isn't there
                    return False
            elif os.path.isfile(local_path):
                self._fs.transfer(local_path, remote_path)
        return True

    def process(self, base, name, extras_dir):
        """
        Process the extras in the release directory.

        :param base: The release directory, containing the feature.
        :param name: The name of the feature.
        :param extras_dir: The directory to move the extras to. A directory will
                           be created within this with the name of the feature.
        :return: True if all extras were transferred successfully; False if the
                 user cancelled a prompt.
        """
        local_dir = self._find_entry(base, self._EXTRAS_SUFFIX)
        logger.debug('Located extras at %s', local_dir)

        remote_dir = os.path.join(extras_dir, name)
        if not self._fs.maybe_mkdir(remote_dir):
            return False

        return self._process_recursive(local_dir, remote_dir)
